"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const is_1 = require("@sindresorhus/is");
const known_hook_events_1 = require("./known-hook-events");
const URLGlobal = typeof URL === 'undefined' ? require('url').URL : URL;
const URLSearchParamsGlobal = typeof URLSearchParams === 'undefined' ? require('url').URLSearchParams : URLSearchParams;
function merge(target, ...sources) {
    for (const source of sources) {
        for (const [key, sourceValue] of Object.entries(source)) {
            if (is_1.default.undefined(sourceValue)) {
                continue;
            }
            const targetValue = target[key];
            if (targetValue instanceof URLSearchParamsGlobal && sourceValue instanceof URLSearchParamsGlobal) {
                const params = new URLSearchParamsGlobal();
                const append = (value, key) => params.append(key, value);
                targetValue.forEach(append);
                sourceValue.forEach(append);
                target[key] = params;
            }
            else if (is_1.default.urlInstance(targetValue) && (is_1.default.urlInstance(sourceValue) || is_1.default.string(sourceValue))) {
                target[key] = new URLGlobal(sourceValue, targetValue);
            }
            else if (is_1.default.plainObject(sourceValue)) {
                if (is_1.default.plainObject(targetValue)) {
                    target[key] = merge({}, targetValue, sourceValue);
                }
                else {
                    target[key] = merge({}, sourceValue);
                }
            }
            else if (is_1.default.array(sourceValue)) {
                target[key] = sourceValue.slice();
            }
            else {
                target[key] = sourceValue;
            }
        }
        if (Reflect.has(source, 'context')) {
            Object.defineProperty(target, 'context', {
                writable: true,
                configurable: true,
                enumerable: false,
                value: source.context
            });
        }
    }
    return target;
}
exports.default = merge;
function mergeOptions(...sources) {
    sources = sources.map(source => {
        if (!source) {
            return {};
        }
        if (is_1.default.object(source.retry)) {
            return source;
        }
        return Object.assign({}, source, { retry: {
                retries: source.retry
            } });
    });
    const mergedOptions = merge({}, ...sources);
    const hooks = known_hook_events_1.default.reduce((accumulator, current) => (Object.assign({}, accumulator, { [current]: [] })), {});
    for (const source of sources) {
        // We need to check `source` to allow calling `.extend()` with no arguments.
        if (source && source.hooks) {
            for (const hook of known_hook_events_1.default) {
                hooks[hook] = hooks[hook].concat(source.hooks[hook] || []);
            }
        }
    }
    mergedOptions.hooks = hooks;
    return mergedOptions;
}
exports.mergeOptions = mergeOptions;
//# sourceMappingURL=merge.js.map